package com.hfkj.service.impl;

import com.hfkj.channel.ccb.CCBTradeService;
import com.hfkj.channel.weixin.WeiXinPayService;
import com.hfkj.dao.BsTradeOrderRefundMapper;
import com.hfkj.entity.*;
import com.hfkj.channel.lakala.LaKaLaTradeService;
import com.hfkj.service.*;
import com.hfkj.sysenum.PlatformTypeEnum;
import com.hfkj.sysenum.StoreDiscountActivityPartakeWayEnum;
import com.hfkj.sysenum.TradeOrderStatusEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service("tradeOrderRefundService")
public class BsTradeOrderRefundServiceImpl implements BsTradeOrderRefundService {

    @Resource
    private BsTradeOrderRefundMapper tradeOrderRefundMapper;

    @Resource
    private BsTradeOrderService tradeOrderService;

    @Resource
    private BsStoreDiscountActivityService storeDiscountActivityService;
    @Resource
    private BsStoreDiscountActivityPartakeUserService discountActivityPartakeUserService;

    @Resource
    private LaKaLaTradeService laKaLaTradeService;

    @Resource
    private CCBTradeService ccbTradeService;

    @Resource
    private UserDiscountService userDiscountService;

    @Resource
    private WeiXinPayService weiXinPayService;

    @Override
    public void editOrderRefund(BsTradeOrderRefund tradeOrderRefund) {
        if (tradeOrderRefund.getId() == null) {
            tradeOrderRefund.setCreateTime(new Date());
            tradeOrderRefund.setUpdateTime(new Date());
            tradeOrderRefundMapper.insert(tradeOrderRefund);
        } else {
            tradeOrderRefund.setUpdateTime(new Date());
            tradeOrderRefundMapper.updateByPrimaryKey(tradeOrderRefund);
        }
    }

    @Override
    public Map<String, Object> tradeOrderRefund(BsTradeOrderRefund tradeOrderRefund) throws Exception {
        editOrderRefund(tradeOrderRefund);

        Map<String, Object> refund = new HashMap<>();
        if (tradeOrderRefund.getPlatformType().equals(PlatformTypeEnum.type1.getNumber())) {
            refund = laKaLaTradeService.refund(tradeOrderRefund.getRefundNo(), tradeOrderRefund.getRefundAmount());

        } else if (tradeOrderRefund.getPlatformType().equals(PlatformTypeEnum.type3.getNumber())) {
            refund = ccbTradeService.refund(tradeOrderRefund.getRefundNo(), tradeOrderRefund.getRefundAmount());

        } else if (tradeOrderRefund.getPlatformType().equals(PlatformTypeEnum.type4.getNumber())) {
            refund = weiXinPayService.refund(tradeOrderRefund.getRefundNo(), tradeOrderRefund.getRefundAmount());
        }

        // 交易订单
        BsTradeOrder tradeOrder = tradeOrderService.getOrderByTradeNoId(tradeOrderRefund.getTradeOrderId());
        if (tradeOrder != null) {
            // 退回门店优惠活动
            if (tradeOrder.getStoreDiscountId() != null) {
                BsStoreDiscountActivity storeDiscountActivity = storeDiscountActivityService.getDetailById(tradeOrder.getStoreDiscountId());
                if (storeDiscountActivity != null
                        && !storeDiscountActivity.getPartakeWay().equals(StoreDiscountActivityPartakeWayEnum.type1.getNumber())
                        && StringUtils.isNotBlank(tradeOrder.getUserPhone())) {

                    BsStoreDiscountActivityPartakeUser partakeUser = discountActivityPartakeUserService.getPartakeUser(tradeOrder.getUserPhone(), tradeOrder.getStoreDiscountId());
                    if (partakeUser != null) {
                        partakeUser.setPartakeSurplusNum(partakeUser.getPartakeSurplusNum() + 1);
                        partakeUser.setPartakeUseNum(partakeUser.getPartakeUseNum() - 1);
                        discountActivityPartakeUserService.editData(partakeUser);
                    }
                }
            }

            if (tradeOrder.getUserDiscountId() != null) {
                userDiscountService.userDiscountReturn(tradeOrder.getUserDiscountId(), tradeOrder.getUserPhone());
            }
        }

        return refund;
    }

    @Override
    public BsTradeOrderRefund getRefundByOutTradeOrder(String outTradeOrder) {
        BsTradeOrderRefundExample example = new BsTradeOrderRefundExample();
        example.createCriteria().andOutTradeNoEqualTo(outTradeOrder);
        List<BsTradeOrderRefund> list = tradeOrderRefundMapper.selectByExample(example);
        if (list.size() > 0) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public BsTradeOrderRefund getRefundByRefundNo(String refundNo) {
        BsTradeOrderRefundExample example = new BsTradeOrderRefundExample();
        example.createCriteria().andRefundNoEqualTo(refundNo);
        List<BsTradeOrderRefund> list = tradeOrderRefundMapper.selectByExample(example);
        if (list.size() > 0) {
            return list.get(0);
        }
        return null;
    }

}
