package com.hfkj.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.utils.DateUtil;
import com.hfkj.common.utils.HttpsUtils;
import com.hfkj.common.utils.IDGenerator;
import com.hfkj.config.CommonSysConst;
import com.hfkj.config.device.SoundService;
import com.hfkj.dao.BsTradeOrderMapper;
import com.hfkj.entity.*;
import com.hfkj.model.UserDiscount;
import com.hfkj.service.*;
import com.hfkj.sysenum.*;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service("tradeOrderService")
public class BsTradeOrderServiceImpl implements BsTradeOrderService {

    @Resource
    private BsTradeOrderMapper tradeOrderMapper;

    @Resource
    private BsMerService merService;

    @Resource
    private BsSalesmanService salesmanService;

    @Resource
    private BsStoreService storeService;

    @Resource
    private BsTradeOrderProfitSharingService tradeOrderProfitSharingService;

    @Resource
    private BsStoreDiscountActivityPartakeUserService discountActivityPartakeUserService;

    @Resource
    private BsDeviceService deviceService;

    @Resource
    private SoundService soundService;

    @Resource
    private UserDiscountService userDiscountService;

    @Override
    public void editOrder(BsTradeOrder tradeOrder) {
        if (tradeOrder.getId() == null) {
            tradeOrder.setCreateTime(new Date());
            tradeOrder.setUpdateTime(new Date());
            tradeOrderMapper.insert(tradeOrder);
        } else {
            tradeOrder.setUpdateTime(new Date());
            tradeOrderMapper.updateByPrimaryKey(tradeOrder);
        }
    }

    @Override
    public Map<String, Object> createOrder(Long storeId, BigDecimal tradeAmount, BsStoreDiscountActivity storeDiscount, String userPhone, UserDiscount userDiscount, Map<String, Object> param) {
        // 查询门店信息
        BsStore store = storeService.getStoreById(storeId);
        if (store == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到门店");
        }
        if (!store.getStatus().equals(StoreStatusEnum.status2.getNumber())) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无法支付！门店状态不正常");
        }
        // 查询商户信息
        BsMer mer = merService.getMer(store.getMerId());
        if (mer == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到商户");
        }
        if (!mer.getMerStatus().equals(MerStatusEnum.status1.getNumber())) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无法支付！商户状态不正常");
        }
        BsTradeOrder tradeOrder = new BsTradeOrder();
        tradeOrder.setCompanyId(mer.getCompanyId());
        tradeOrder.setAgentId(mer.getAgentId());
        tradeOrder.setSalesmanId(mer.getSalesmanId());
        // 业务人员
        BsSalesman salesman = salesmanService.getSalesman(mer.getSalesmanId());
        if (salesman != null) {
            tradeOrder.setSalesmanName(salesman.getName());
        }
        tradeOrder.setMerId(store.getMerId());
        tradeOrder.setMerNo(mer.getMerNo());
        tradeOrder.setMerName(mer.getMerAbbreviate());
        tradeOrder.setStoreId(storeId);
        tradeOrder.setStoreName(store.getName());
        tradeOrder.setStatus(TradeOrderStatusEnum.status1.getNumber());
        tradeOrder.setTradeAmount(tradeAmount);
        tradeOrder.setTradeActualAmount(tradeAmount);
        tradeOrder.setStoreDiscountSatisfy(false);
        tradeOrder.setUserPhone(userPhone);


        // 门店优惠
        if (storeDiscount != null) {
            tradeOrder.setStoreDiscountId(storeDiscount.getId());
            tradeOrder.setStoreDiscountName(storeDiscount.getStoreName());
            tradeOrder.setStoreDiscountType(storeDiscount.getDiscountType());
            tradeOrder.setStoreDiscountCondition(storeDiscount.getDiscountCondition());
            tradeOrder.setStoreDiscountPrice(storeDiscount.getDiscountPrice());
            if (storeDiscount.getDiscountType().equals(StoreDiscountActivityTypeEnum.type1.getNumber())) {
                // 未达到满减门槛
                if (userDiscount.getDiscountCondition().compareTo(tradeOrder.getTradeAmount()) >= 0) {
                    tradeOrder.setStoreDiscountActualPrice(new BigDecimal(0));
                } else {
                    tradeOrder.setStoreDiscountSatisfy(true);
                    tradeOrder.setStoreDiscountActualPrice(storeDiscount.getDiscountPrice());
                }
                tradeOrder.setTradeActualAmount(tradeOrder.getTradeActualAmount().subtract(tradeOrder.getStoreDiscountActualPrice()));

            } else if (storeDiscount.getDiscountType().equals(StoreDiscountActivityTypeEnum.type3.getNumber())) {
                // 优惠比例 / 100 = 最终优惠比例
                tradeOrder.setStoreDiscountSatisfy(true);
                BigDecimal priceRate = tradeOrder.getStoreDiscountPrice().divide(new BigDecimal("100").setScale(2, BigDecimal.ROUND_DOWN));
                tradeOrder.setStoreDiscountActualPrice(tradeOrder.getTradeActualAmount().subtract(tradeOrder.getTradeActualAmount().multiply(priceRate).setScale(2, BigDecimal.ROUND_HALF_UP)));
                tradeOrder.setTradeActualAmount(tradeOrder.getTradeActualAmount().subtract(tradeOrder.getStoreDiscountActualPrice()));
            } else {
                tradeOrder.setStoreDiscountSatisfy(true);
                tradeOrder.setStoreDiscountActualPrice(storeDiscount.getDiscountPrice());
                tradeOrder.setTradeActualAmount(tradeOrder.getTradeActualAmount().subtract(tradeOrder.getStoreDiscountActualPrice()));
            }
        }

        // 用户优惠券
        if (userDiscount != null) {
            tradeOrder.setUserDiscountId(userDiscount.getId());
            tradeOrder.setUserDiscountName(userDiscount.getDiscountName());
            tradeOrder.setUserDiscountType(userDiscount.getDiscountType());
            tradeOrder.setUserDiscountPrice(userDiscount.getDiscountPrice());

            if (userDiscount.getDiscountType().equals(UserDiscountTypeEnum.type1.getNumber())) {
                if (userDiscount.getDiscountCondition().compareTo(tradeOrder.getTradeAmount()) > 0) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "优惠券未达到满减门槛");
                }
                tradeOrder.setUserDiscountActualPrice(userDiscount.getDiscountPrice());
                tradeOrder.setTradeActualAmount(tradeOrder.getTradeActualAmount().subtract(tradeOrder.getUserDiscountActualPrice()));

            } else if (userDiscount.getDiscountType().equals(UserDiscountTypeEnum.type3.getNumber())) {
                // 优惠比例 / 100 = 最终优惠比例
                BigDecimal priceRate = tradeOrder.getUserDiscountPrice().setScale(2, BigDecimal.ROUND_DOWN);
                tradeOrder.setUserDiscountActualPrice(tradeOrder.getTradeActualAmount().subtract(tradeOrder.getTradeActualAmount().multiply(priceRate).setScale(2, BigDecimal.ROUND_HALF_UP)));
                tradeOrder.setTradeActualAmount(tradeOrder.getTradeActualAmount().subtract(tradeOrder.getUserDiscountActualPrice()));
            } else {
                tradeOrder.setUserDiscountActualPrice(userDiscount.getDiscountPrice());
                tradeOrder.setTradeActualAmount(tradeOrder.getTradeActualAmount().subtract(tradeOrder.getUserDiscountActualPrice()));
            }
        }

        // 生成订单号
        String tradeNo = "ZF1130"+DateUtil.format(new Date(), DateUtil.YMDHMS) + IDGenerator.nextId(10);
        tradeOrder.setOutTradeNo(tradeNo);

        if (StringUtils.isNotBlank(MapUtils.getString(param, "payMode"))) {
            tradeOrder.setPayMode(MapUtils.getString(param, "payMode"));
        }
        if (StringUtils.isNotBlank(MapUtils.getString(param, "payUserId"))) {
            tradeOrder.setPayUserId(MapUtils.getString(param, "payUserId"));
        }

        if (StringUtils.isNotBlank(MapUtils.getString(param, "deviceSN"))) {
            tradeOrder.setDeviceSn(MapUtils.getString(param, "deviceSN"));
        }

        if (MapUtils.getLong(param, "deviceId") != null) {
            tradeOrder.setDeviceId(MapUtils.getLong(param, "deviceId"));
        }

        if (StringUtils.isNotBlank(MapUtils.getString(param, "qrCodeSN"))) {
            tradeOrder.setQrCodeSn(MapUtils.getString(param, "qrCodeSN"));
        }

        if (MapUtils.getLong(param, "qrCodeId") != null) {
            tradeOrder.setQrCodeId(MapUtils.getLong(param, "qrCodeId"));
        }

        tradeOrder.setProfitSharingStatus(true);
        editOrder(tradeOrder);

        Map<String, Object> returnParam = new HashMap<>();
        returnParam.put("outTradeNo", tradeNo);
        return returnParam;
    }

    @Override
    public void paySuccess(String outTradeNo, String transactionId, PlatformTypeEnum platformType, TradeOrderPayModeEnum tradeOrderPayMode, BigDecimal tradeActualAmount, String subMchId) {
        // 查询订单
        BsTradeOrder tradeOrder = getOrderByOutTradeNo(outTradeNo);
        if (tradeOrder != null && tradeOrder.getStatus().equals(TradeOrderStatusEnum.status1.getNumber())) {
            tradeOrder.setPayMode(tradeOrderPayMode.getCode());
            tradeOrder.setPlatformType(platformType.getNumber());
            tradeOrder.setPlatformMerNo(subMchId);
            tradeOrder.setPlatformTradeNo(transactionId);
            tradeOrder.setTradeActualAmount(tradeActualAmount);
            tradeOrder.setStatus(TradeOrderStatusEnum.status3.getNumber());
            tradeOrder.setPayTime(new Date());
            editOrder(tradeOrder);

            if (tradeOrder.getStoreDiscountId() != null && StringUtils.isNotBlank(tradeOrder.getUserPhone())) {
                BsStoreDiscountActivityPartakeUser partakeUser = discountActivityPartakeUserService.getPartakeUser(tradeOrder.getUserPhone(), tradeOrder.getStoreDiscountId());
                if (partakeUser != null) {
                    partakeUser.setPartakeSurplusNum(partakeUser.getPartakeSurplusNum() - 1);
                    partakeUser.setPartakeUseNum(partakeUser.getPartakeUseNum() + 1);
                    discountActivityPartakeUserService.editData(partakeUser);
                }
            }

            if (tradeOrder.getUserDiscountId() != null && StringUtils.isNotBlank(tradeOrder.getUserPhone())) {
                userDiscountService.userDiscountUse(tradeOrder.getUserDiscountId(), tradeOrder.getUserPhone());
            }

            // 创建分账
            if (tradeOrder.getProfitSharingStatus().equals(true)) {
                tradeOrderProfitSharingService.createProfitSharing(tradeOrder);
            }

            Map<String,Object> param = new HashMap<>();
            param.put("storeId", tradeOrder.getStoreId());
            param.put("deviceType", DeviceTypeEnum.type1.getNumber());
            param.put("status", DeviceStatusEnum.status1.getNumber());
            List<BsDevice> deviceList = deviceService.getDeviceList(param);
            for (BsDevice device : deviceList) {
                soundService.sendPayMsg(device.getDeviceNo(), tradeOrder.getTradeAmount());
            }
        }
    }

    @Override
    public BsTradeOrder getOrderByOutTradeNo(String outTradeNo) {
        BsTradeOrderExample example = new BsTradeOrderExample();
        example.createCriteria().andOutTradeNoEqualTo(outTradeNo);
        List<BsTradeOrder> list = tradeOrderMapper.selectByExample(example);
        if (list.size() > 0) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public BsTradeOrder getOrderByTradeNoId(Long tradeOrderId) {
        return tradeOrderMapper.selectByPrimaryKey(tradeOrderId);
    }

    @Override
    public List<BsTradeOrder> getTradeOrderList(Map<String, Object> param) {
        BsTradeOrderExample example = new BsTradeOrderExample();
        BsTradeOrderExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(param, "agentId") != null) {
            criteria.andAgentIdEqualTo(MapUtils.getLong(param, "agentId"));
        }

        if (MapUtils.getLong(param, "salesmanId") != null) {
            criteria.andSalesmanIdEqualTo(MapUtils.getLong(param, "salesmanId"));
        }

        if (MapUtils.getLong(param, "merId") != null) {
            criteria.andMerIdEqualTo(MapUtils.getLong(param, "merId"));
        }

        if (StringUtils.isNotBlank(MapUtils.getString(param, "merName"))) {
            criteria.andMerNameLike("%"+MapUtils.getString(param, "merName")+"%");
        }

        if (MapUtils.getLong(param, "storeId") != null) {
            criteria.andStoreIdEqualTo(MapUtils.getLong(param, "storeId"));
        }

        if (StringUtils.isNotBlank(MapUtils.getString(param, "storeName"))) {
            criteria.andStoreNameLike("%"+MapUtils.getString(param, "storeName")+"%");
        }

        if (StringUtils.isNotBlank(MapUtils.getString(param, "payMode"))) {
            criteria.andPayModeEqualTo(MapUtils.getString(param, "payMode"));
        }

        if (StringUtils.isNotBlank(MapUtils.getString(param, "outTradeNo"))) {
            criteria.andOutTradeNoEqualTo(MapUtils.getString(param, "outTradeNo"));
        }

        if (StringUtils.isNotBlank(MapUtils.getString(param, "accTradeNo"))) {
            criteria.andAccTradeNoEqualTo(MapUtils.getString(param, "accTradeNo"));
        }

        if (MapUtils.getLong(param, "payTimeS") != null) {
            criteria.andPayTimeGreaterThanOrEqualTo(new Date(MapUtils.getLong(param, "payTimeS")));
        }

        if (MapUtils.getLong(param, "payTimeE") != null) {
            criteria.andPayTimeLessThanOrEqualTo(new Date(MapUtils.getLong(param, "payTimeE")));
        }

        if (MapUtils.getInteger(param, "platformType") != null) {
            criteria.andPlatformTypeEqualTo(MapUtils.getInteger(param, "platformType"));
        }

        if (MapUtils.getInteger(param, "status") != null) {
            criteria.andStatusEqualTo(MapUtils.getInteger(param, "status"));
        }

        example.setOrderByClause("create_time desc");
        return tradeOrderMapper.selectByExample(example);
    }

    @Override
    public BigDecimal countTradeOrderPrice(Map<String, Object> param) {
        return tradeOrderMapper.countTradeOrderPrice(param);
    }
}
